# Generated by Django 3.2.13 on 2022-06-01 22:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vendor', '0032_alter_invoice_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='payment',
            name='status',
            field=models.IntegerField(choices=[(1, 'Queued'), (2, 'Active'), (10, 'Authorized'), (15, 'Captured'), (20, 'Settled'), (30, 'Canceled'), (35, 'Refunded'), (40, 'Declined'), (50, 'Void')], default=0, verbose_name='Status'),
        ),
    ]



from django.core.exceptions import ImproperlyConfigured
from django.contrib import messages
from django.http import Http404
from django.shortcuts import redirect
from django.utils.translation import gettext as _
from vendor.utils import get_site_from_request


class PassRequestToFormKwargsMixin:

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['request'] = self.request
        return kwargs


class ProductRequiredMixin:
    """
    Checks to see if a user has a required product and if not, redirects them.
    ProductRequiredMixin expects that request.site is set.
    """
    product_queryset = None
    product_model = None
    product_redirect = "/"
    product_owned = False

    def dispatch(self, request, *args, **kwargs):
        """
        Verify that the current user owns the product.  Checks on all HTTP methods.
        """
        if not self.user_has_product():
            return self.handle_no_product()
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # Variable set by ProductRequiredMixin
        context['product_owned'] = self.product_owned
        return context

    def user_has_product(self):
        """
        Check to see if a user has a viable product license based on the get_product_queryset() method.

        TODO: move this to some kind of caching or session variable to avoid hitting the DB on every request.
        """
        if self.request.user.is_anonymous:
            self.product_owned = False
        else:
            products = self.get_product_queryset()
            self.product_owned = self.request.user.customer_profile.filter(
                site=get_site_from_request(self.request)).get().has_product(products)

        return self.product_owned

    def get_product_queryset(self):
        """
        Method to get the Product(s) needed for the check.  Can be overridden to handle complex queries.
        """
        if self.product_queryset is None:
            if self.product_model:
                # Only provide list of products on the current site.
                return self.product_model.objects.filter(site=get_site_from_request(self.request))
            else:
                raise ImproperlyConfigured(
                    "%(cls)s is missing a Product QuerySet. Define "
                    "%(cls)s.product_model, %(cls)s.product_queryset, or override "
                    "%(cls)s.get_product_queryset()." % {
                        'cls': self.__class__.__name__
                    }
                )
        return self.product_queryset.all()

    def handle_no_product(self):
        '''
        The result to be returned if no viable product is found owned by the user.
        If None is returned, default to a 404 response.

        This method allows for a dynamic redirect to be used.
        Default is to use the product_redirect path.
        '''

        if not self.product_redirect:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': self.get_product_queryset().model._meta.verbose_name})

        messages.info(self.request, _("Product Purchase required."))
        return redirect(self.product_redirect)


class SiteOnRequestFilterMixin:

    def get_queryset(self):
        if hasattr(self.request, 'site'):
            return self.model.objects.filter(site=get_site_from_request(self.request))
        return self.model.on_site.all()


class TableFilterMixin:
    """
    Mixin for List Views that want to add dynamica pagination, ordering and search filters.
    The mixin expect paginate_by, ordering and search_filter in the request.GET parameter.
    """
    paginate_by = None
    ordering = []
    search_filter = None

    def search_filter(self, queryset):
        """
        Override this funtion in the ListView to filter wanted fields for the model
        defined in the ListView
        """
        # Override funtion in List View to filter wanted model fields
        return queryset

    def set_paginate_by(self):
        """
        Set the self.paginate_by variable from the Pagination Class in the ListView
        """
        if self.request.GET.get('paginate_by'):
            self.paginate_by = self.request.GET.get('paginate_by')

    def set_ordering(self):
        """
        Sets a ordering list that can be intrepreted by the order_by() query method.
        """
        self.ordering = []
        if self.request.GET.get('ordering', []):
            self.ordering = [order for order in self.request.GET.get('ordering', []).split(',') if order]

    def get_search_filter_queryset(self, queryset):
        """
        If search_filter is in the request.GET parameters it will call search_filter and return
        a queryset object.
        """
        if self.request.GET.get('search_filter'):
            return self.search_filter(queryset)
        return queryset

    def get_queryset(self):
        queryset = super().get_queryset()

        self.set_paginate_by()

        self.set_ordering()

        queryset = self.get_search_filter_queryset(queryset)

        return queryset.order_by(*self.ordering)



# Generated by Django 3.1.1 on 2020-09-10 22:06

import autoslug.fields
from django.conf import settings
import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager
import uuid

from vendor.config import VENDOR_PRODUCT_MODEL

class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(VENDOR_PRODUCT_MODEL),
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='Home', max_length=80, verbose_name='Address Name')),
                ('address_1', models.CharField(max_length=40, verbose_name='Address 1')),
                ('address_2', models.CharField(blank=True, max_length=40, null=True, verbose_name='Address 2')),
                ('locality', models.CharField(max_length=40, verbose_name='City')),
                ('state', models.CharField(max_length=40, verbose_name='State')),
                ('country', models.IntegerField(choices=[(581, 'United States')], default=581, verbose_name='Country')),
                ('postal_code', models.CharField(blank=True, max_length=16, verbose_name='Postal Code')),
            ],
        ),
        migrations.CreateModel(
            name='CustomerProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last updated')),
                ('currency', models.CharField(choices=[('afn', 'AFN'), ('eur', 'EUR'), ('all', 'ALL'), ('dzd', 'DZD'), ('usd', 'USD'), ('aoa', 'AOA'), ('xcd', 'XCD'), ('ars', 'ARS'), ('amd', 'AMD'), ('awg', 'AWG'), ('aud', 'AUD'), ('azn', 'AZN'), ('bsd', 'BSD'), ('bhd', 'BHD'), ('bdt', 'BDT'), ('bbd', 'BBD'), ('byn', 'BYN'), ('bzd', 'BZD'), ('xof', 'XOF'), ('bmd', 'BMD'), ('inr', 'INR'), ('btn', 'BTN'), ('bob', 'BOB'), ('bov', 'BOV'), ('bam', 'BAM'), ('bwp', 'BWP'), ('nok', 'NOK'), ('brl', 'BRL'), ('bnd', 'BND'), ('bgn', 'BGN'), ('bif', 'BIF'), ('cve', 'CVE'), ('khr', 'KHR'), ('xaf', 'XAF'), ('cad', 'CAD'), ('kyd', 'KYD'), ('clp', 'CLP'), ('clf', 'CLF'), ('cny', 'CNY'), ('cop', 'COP'), ('cou', 'COU'), ('kmf', 'KMF'), ('cdf', 'CDF'), ('nzd', 'NZD'), ('crc', 'CRC'), ('hrk', 'HRK'), ('cup', 'CUP'), ('cuc', 'CUC'), ('ang', 'ANG'), ('czk', 'CZK'), ('dkk', 'DKK'), ('djf', 'DJF'), ('dop', 'DOP'), ('egp', 'EGP'), ('svc', 'SVC'), ('ern', 'ERN'), ('etb', 'ETB'), ('fkp', 'FKP'), ('fjd', 'FJD'), ('xpf', 'XPF'), ('gmd', 'GMD'), ('gel', 'GEL'), ('ghs', 'GHS'), ('gip', 'GIP'), ('gtq', 'GTQ'), ('gbp', 'GBP'), ('gnf', 'GNF'), ('gyd', 'GYD'), ('htg', 'HTG'), ('hnl', 'HNL'), ('hkd', 'HKD'), ('huf', 'HUF'), ('isk', 'ISK'), ('idr', 'IDR'), ('irr', 'IRR'), ('iqd', 'IQD'), ('ils', 'ILS'), ('jmd', 'JMD'), ('jpy', 'JPY'), ('jod', 'JOD'), ('kzt', 'KZT'), ('kes', 'KES'), ('kpw', 'KPW'), ('krw', 'KRW'), ('kwd', 'KWD'), ('kgs', 'KGS'), ('lak', 'LAK'), ('lbp', 'LBP'), ('lsl', 'LSL'), ('zar', 'ZAR'), ('lrd', 'LRD'), ('lyd', 'LYD'), ('chf', 'CHF'), ('mop', 'MOP'), ('mkd', 'MKD'), ('mga', 'MGA'), ('mwk', 'MWK'), ('myr', 'MYR'), ('mvr', 'MVR'), ('mru', 'MRU'), ('mur', 'MUR'), ('mxn', 'MXN'), ('mxv', 'MXV'), ('mdl', 'MDL'), ('mnt', 'MNT'), ('mad', 'MAD'), ('mzn', 'MZN'), ('mmk', 'MMK'), ('nad', 'NAD'), ('npr', 'NPR'), ('nio', 'NIO'), ('ngn', 'NGN'), ('omr', 'OMR'), ('pkr', 'PKR'), ('pab', 'PAB'), ('pgk', 'PGK'), ('pyg', 'PYG'), ('pen', 'PEN'), ('php', 'PHP'), ('pln', 'PLN'), ('qar', 'QAR'), ('ron', 'RON'), ('rub', 'RUB'), ('rwf', 'RWF'), ('shp', 'SHP'), ('wst', 'WST'), ('stn', 'STN'), ('sar', 'SAR'), ('rsd', 'RSD'), ('scr', 'SCR'), ('sll', 'SLL'), ('sgd', 'SGD'), ('sbd', 'SBD'), ('sos', 'SOS'), ('ssp', 'SSP'), ('lkr', 'LKR'), ('sdg', 'SDG'), ('srd', 'SRD'), ('szl', 'SZL'), ('sek', 'SEK'), ('che', 'CHE'), ('chw', 'CHW'), ('syp', 'SYP'), ('twd', 'TWD'), ('tjs', 'TJS'), ('tzs', 'TZS'), ('thb', 'THB'), ('top', 'TOP'), ('ttd', 'TTD'), ('tnd', 'TND'), ('try', 'TRY'), ('tmt', 'TMT'), ('ugx', 'UGX'), ('uah', 'UAH'), ('aed', 'AED'), ('usn', 'USN'), ('uyu', 'UYU'), ('uyi', 'UYI'), ('uyw', 'UYW'), ('uzs', 'UZS'), ('vuv', 'VUV'), ('ves', 'VES'), ('vnd', 'VND'), ('yer', 'YER'), ('zmw', 'ZMW'), ('zwl', 'ZWL')], default='usd', max_length=4, verbose_name='Currency')),
                ('site', models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='customer_profile', to='sites.site')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='customer_profile', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Customer Profile',
                'verbose_name_plural': 'Customer Profiles',
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('on_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last updated')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='UUID')),
                ('status', models.IntegerField(choices=[(0, 'Cart'), (10, 'Checkout'), (20, 'Queued'), (30, 'Processing'), (40, 'Failed'), (50, 'Complete'), (60, 'Refunded')], default=0, verbose_name='Status')),
                ('customer_notes', models.JSONField(blank=True, default=dict, null=True, verbose_name='Customer Notes')),
                ('vendor_notes', models.JSONField(blank=True, default=dict, null=True, verbose_name='Vendor Notes')),
                ('ordered_date', models.DateTimeField(blank=True, null=True, verbose_name='Ordered Date')),
                ('subtotal', models.FloatField(default=0.0)),
                ('tax', models.FloatField(blank=True, null=True)),
                ('shipping', models.FloatField(blank=True, null=True)),
                ('total', models.FloatField(blank=True, null=True)),
                ('currency', models.CharField(choices=[('afn', 'AFN'), ('eur', 'EUR'), ('all', 'ALL'), ('dzd', 'DZD'), ('usd', 'USD'), ('aoa', 'AOA'), ('xcd', 'XCD'), ('ars', 'ARS'), ('amd', 'AMD'), ('awg', 'AWG'), ('aud', 'AUD'), ('azn', 'AZN'), ('bsd', 'BSD'), ('bhd', 'BHD'), ('bdt', 'BDT'), ('bbd', 'BBD'), ('byn', 'BYN'), ('bzd', 'BZD'), ('xof', 'XOF'), ('bmd', 'BMD'), ('inr', 'INR'), ('btn', 'BTN'), ('bob', 'BOB'), ('bov', 'BOV'), ('bam', 'BAM'), ('bwp', 'BWP'), ('nok', 'NOK'), ('brl', 'BRL'), ('bnd', 'BND'), ('bgn', 'BGN'), ('bif', 'BIF'), ('cve', 'CVE'), ('khr', 'KHR'), ('xaf', 'XAF'), ('cad', 'CAD'), ('kyd', 'KYD'), ('clp', 'CLP'), ('clf', 'CLF'), ('cny', 'CNY'), ('cop', 'COP'), ('cou', 'COU'), ('kmf', 'KMF'), ('cdf', 'CDF'), ('nzd', 'NZD'), ('crc', 'CRC'), ('hrk', 'HRK'), ('cup', 'CUP'), ('cuc', 'CUC'), ('ang', 'ANG'), ('czk', 'CZK'), ('dkk', 'DKK'), ('djf', 'DJF'), ('dop', 'DOP'), ('egp', 'EGP'), ('svc', 'SVC'), ('ern', 'ERN'), ('etb', 'ETB'), ('fkp', 'FKP'), ('fjd', 'FJD'), ('xpf', 'XPF'), ('gmd', 'GMD'), ('gel', 'GEL'), ('ghs', 'GHS'), ('gip', 'GIP'), ('gtq', 'GTQ'), ('gbp', 'GBP'), ('gnf', 'GNF'), ('gyd', 'GYD'), ('htg', 'HTG'), ('hnl', 'HNL'), ('hkd', 'HKD'), ('huf', 'HUF'), ('isk', 'ISK'), ('idr', 'IDR'), ('irr', 'IRR'), ('iqd', 'IQD'), ('ils', 'ILS'), ('jmd', 'JMD'), ('jpy', 'JPY'), ('jod', 'JOD'), ('kzt', 'KZT'), ('kes', 'KES'), ('kpw', 'KPW'), ('krw', 'KRW'), ('kwd', 'KWD'), ('kgs', 'KGS'), ('lak', 'LAK'), ('lbp', 'LBP'), ('lsl', 'LSL'), ('zar', 'ZAR'), ('lrd', 'LRD'), ('lyd', 'LYD'), ('chf', 'CHF'), ('mop', 'MOP'), ('mkd', 'MKD'), ('mga', 'MGA'), ('mwk', 'MWK'), ('myr', 'MYR'), ('mvr', 'MVR'), ('mru', 'MRU'), ('mur', 'MUR'), ('mxn', 'MXN'), ('mxv', 'MXV'), ('mdl', 'MDL'), ('mnt', 'MNT'), ('mad', 'MAD'), ('mzn', 'MZN'), ('mmk', 'MMK'), ('nad', 'NAD'), ('npr', 'NPR'), ('nio', 'NIO'), ('ngn', 'NGN'), ('omr', 'OMR'), ('pkr', 'PKR'), ('pab', 'PAB'), ('pgk', 'PGK'), ('pyg', 'PYG'), ('pen', 'PEN'), ('php', 'PHP'), ('pln', 'PLN'), ('qar', 'QAR'), ('ron', 'RON'), ('rub', 'RUB'), ('rwf', 'RWF'), ('shp', 'SHP'), ('wst', 'WST'), ('stn', 'STN'), ('sar', 'SAR'), ('rsd', 'RSD'), ('scr', 'SCR'), ('sll', 'SLL'), ('sgd', 'SGD'), ('sbd', 'SBD'), ('sos', 'SOS'), ('ssp', 'SSP'), ('lkr', 'LKR'), ('sdg', 'SDG'), ('srd', 'SRD'), ('szl', 'SZL'), ('sek', 'SEK'), ('che', 'CHE'), ('chw', 'CHW'), ('syp', 'SYP'), ('twd', 'TWD'), ('tjs', 'TJS'), ('tzs', 'TZS'), ('thb', 'THB'), ('top', 'TOP'), ('ttd', 'TTD'), ('tnd', 'TND'), ('try', 'TRY'), ('tmt', 'TMT'), ('ugx', 'UGX'), ('uah', 'UAH'), ('aed', 'AED'), ('usn', 'USN'), ('uyu', 'UYU'), ('uyi', 'UYI'), ('uyw', 'UYW'), ('uzs', 'UZS'), ('vuv', 'VUV'), ('ves', 'VES'), ('vnd', 'VND'), ('yer', 'YER'), ('zmw', 'ZMW'), ('zwl', 'ZWL')], default='usd', max_length=4, verbose_name='Currency')),
                ('profile', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoices', to='vendor.customerprofile', verbose_name='Customer Profile')),
                ('shipping_address', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='vendor.address', verbose_name='Shipping Address')),
                ('site', models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='invoices', to='sites.site')),
            ],
            options={
                'verbose_name': 'Invoice',
                'verbose_name_plural': 'Invoices',
                'ordering': ['-ordered_date', '-updated'],
                'permissions': (('can_view_site_purchases', 'Can view Site Purchases'), ('can_refund_purchase', 'Can refund Purchase')),
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('on_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='Offer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last updated')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from='name', unique_with=('site__id',))),
                ('name', models.CharField(blank=True, max_length=80, verbose_name='Name')),
                ('start_date', models.DateTimeField(help_text='What date should this offer become available?', verbose_name='Start Date')),
                ('end_date', models.DateTimeField(blank=True, help_text='Expiration Date?', null=True, verbose_name='End Date')),
                ('terms', models.IntegerField(choices=[(0, 'Perpetual'), (10, 'Subscription'), (20, 'One-Time Use')], default=0, verbose_name='Terms')),
                ('term_details', models.JSONField(blank=True, default=dict, null=True, verbose_name='Term Details')),
                ('term_start_date', models.DateTimeField(blank=True, help_text='When is this product available to use?', null=True, verbose_name='Term Start Date')),
                ('available', models.BooleanField(default=False, help_text='Is this currently available?', verbose_name='Available')),
                ('bundle', models.BooleanField(default=False, help_text='Is this a product bundle? (auto-generated)', verbose_name='Is a Bundle?')),
                ('site', models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='product_offers', to='sites.site')),
            ],
            options={
                'verbose_name': 'Offer',
                'verbose_name_plural': 'Offers',
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('on_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='OrderItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last updated')),
                ('quantity', models.IntegerField(default=1, verbose_name='Quantity')),
                ('invoice', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='order_items', to='vendor.invoice', verbose_name='Invoice')),
                ('offer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='order_items', to='vendor.offer', verbose_name='Offer')),
            ],
            options={
                'verbose_name': 'Order Item',
                'verbose_name_plural': 'Order Items',
            },
        ),
        migrations.CreateModel(
            name='TaxClassifier',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=80, verbose_name='Name')),
                ('taxable', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Product Classifier',
                'verbose_name_plural': 'Product Classifiers',
            },
        ),
        migrations.CreateModel(
            name='Wishlist',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('profile', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='wishlists', to='vendor.customerprofile', verbose_name='Purchase Profile')),
            ],
            options={
                'verbose_name': 'Wishlist',
                'verbose_name_plural': 'Wishlists',
            },
        ),
        migrations.CreateModel(
            name='WishlistItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last updated')),
                ('offer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='wishlist_items', to='vendor.offer', verbose_name='Offer')),
                ('wishlist', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='wishlist_items', to='vendor.wishlist', verbose_name='Wishlist')),
            ],
            options={
                'verbose_name': 'Wishlist Item',
                'verbose_name_plural': 'Wishlist Items',
            },
        ),
        migrations.CreateModel(
            name='Receipt',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last updated')),
                ('start_date', models.DateTimeField(blank=True, null=True, verbose_name='Start Date')),
                ('end_date', models.DateTimeField(blank=True, null=True, verbose_name='End Date')),
                ('auto_renew', models.BooleanField(default=False, verbose_name='Auto Renew')),
                ('vendor_notes', models.JSONField(default=dict, verbose_name='Vendor Notes')),
                ('transaction', models.CharField(max_length=80, verbose_name='Transaction')),
                ('status', models.IntegerField(choices=[(1, 'Queued'), (2, 'Active'), (10, 'Authorized'), (15, 'Captured'), (20, 'Completed'), (30, 'Canceled'), (35, 'Refunded')], default=0, verbose_name='Status')),
                ('meta', models.JSONField(default=dict)),
                ('order_item', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='receipts', to='vendor.orderitem', verbose_name='Order Item')),
                ('profile', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='receipts', to='vendor.customerprofile', verbose_name='Purchase Profile')),
            ],
            options={
                'verbose_name': 'Receipt',
                'verbose_name_plural': 'Receipts',
            },
        ),
        migrations.CreateModel(
            name='Price',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('cost', models.FloatField(blank=True, null=True)),
                ('currency', models.CharField(choices=[('afn', 'AFN'), ('eur', 'EUR'), ('all', 'ALL'), ('dzd', 'DZD'), ('usd', 'USD'), ('aoa', 'AOA'), ('xcd', 'XCD'), ('ars', 'ARS'), ('amd', 'AMD'), ('awg', 'AWG'), ('aud', 'AUD'), ('azn', 'AZN'), ('bsd', 'BSD'), ('bhd', 'BHD'), ('bdt', 'BDT'), ('bbd', 'BBD'), ('byn', 'BYN'), ('bzd', 'BZD'), ('xof', 'XOF'), ('bmd', 'BMD'), ('inr', 'INR'), ('btn', 'BTN'), ('bob', 'BOB'), ('bov', 'BOV'), ('bam', 'BAM'), ('bwp', 'BWP'), ('nok', 'NOK'), ('brl', 'BRL'), ('bnd', 'BND'), ('bgn', 'BGN'), ('bif', 'BIF'), ('cve', 'CVE'), ('khr', 'KHR'), ('xaf', 'XAF'), ('cad', 'CAD'), ('kyd', 'KYD'), ('clp', 'CLP'), ('clf', 'CLF'), ('cny', 'CNY'), ('cop', 'COP'), ('cou', 'COU'), ('kmf', 'KMF'), ('cdf', 'CDF'), ('nzd', 'NZD'), ('crc', 'CRC'), ('hrk', 'HRK'), ('cup', 'CUP'), ('cuc', 'CUC'), ('ang', 'ANG'), ('czk', 'CZK'), ('dkk', 'DKK'), ('djf', 'DJF'), ('dop', 'DOP'), ('egp', 'EGP'), ('svc', 'SVC'), ('ern', 'ERN'), ('etb', 'ETB'), ('fkp', 'FKP'), ('fjd', 'FJD'), ('xpf', 'XPF'), ('gmd', 'GMD'), ('gel', 'GEL'), ('ghs', 'GHS'), ('gip', 'GIP'), ('gtq', 'GTQ'), ('gbp', 'GBP'), ('gnf', 'GNF'), ('gyd', 'GYD'), ('htg', 'HTG'), ('hnl', 'HNL'), ('hkd', 'HKD'), ('huf', 'HUF'), ('isk', 'ISK'), ('idr', 'IDR'), ('irr', 'IRR'), ('iqd', 'IQD'), ('ils', 'ILS'), ('jmd', 'JMD'), ('jpy', 'JPY'), ('jod', 'JOD'), ('kzt', 'KZT'), ('kes', 'KES'), ('kpw', 'KPW'), ('krw', 'KRW'), ('kwd', 'KWD'), ('kgs', 'KGS'), ('lak', 'LAK'), ('lbp', 'LBP'), ('lsl', 'LSL'), ('zar', 'ZAR'), ('lrd', 'LRD'), ('lyd', 'LYD'), ('chf', 'CHF'), ('mop', 'MOP'), ('mkd', 'MKD'), ('mga', 'MGA'), ('mwk', 'MWK'), ('myr', 'MYR'), ('mvr', 'MVR'), ('mru', 'MRU'), ('mur', 'MUR'), ('mxn', 'MXN'), ('mxv', 'MXV'), ('mdl', 'MDL'), ('mnt', 'MNT'), ('mad', 'MAD'), ('mzn', 'MZN'), ('mmk', 'MMK'), ('nad', 'NAD'), ('npr', 'NPR'), ('nio', 'NIO'), ('ngn', 'NGN'), ('omr', 'OMR'), ('pkr', 'PKR'), ('pab', 'PAB'), ('pgk', 'PGK'), ('pyg', 'PYG'), ('pen', 'PEN'), ('php', 'PHP'), ('pln', 'PLN'), ('qar', 'QAR'), ('ron', 'RON'), ('rub', 'RUB'), ('rwf', 'RWF'), ('shp', 'SHP'), ('wst', 'WST'), ('stn', 'STN'), ('sar', 'SAR'), ('rsd', 'RSD'), ('scr', 'SCR'), ('sll', 'SLL'), ('sgd', 'SGD'), ('sbd', 'SBD'), ('sos', 'SOS'), ('ssp', 'SSP'), ('lkr', 'LKR'), ('sdg', 'SDG'), ('srd', 'SRD'), ('szl', 'SZL'), ('sek', 'SEK'), ('che', 'CHE'), ('chw', 'CHW'), ('syp', 'SYP'), ('twd', 'TWD'), ('tjs', 'TJS'), ('tzs', 'TZS'), ('thb', 'THB'), ('top', 'TOP'), ('ttd', 'TTD'), ('tnd', 'TND'), ('try', 'TRY'), ('tmt', 'TMT'), ('ugx', 'UGX'), ('uah', 'UAH'), ('aed', 'AED'), ('usn', 'USN'), ('uyu', 'UYU'), ('uyi', 'UYI'), ('uyw', 'UYW'), ('uzs', 'UZS'), ('vuv', 'VUV'), ('ves', 'VES'), ('vnd', 'VND'), ('yer', 'YER'), ('zmw', 'ZMW'), ('zwl', 'ZWL')], default='usd', max_length=4, verbose_name='Currency')),
                ('start_date', models.DateTimeField(help_text='When should the price first become available?', verbose_name='Start Date')),
                ('end_date', models.DateTimeField(blank=True, help_text='When should the price expire?', null=True, verbose_name='End Date')),
                ('priority', models.IntegerField(blank=True, help_text='Higher number takes priority', null=True, verbose_name='Priority')),
                ('offer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='prices', to='vendor.offer')),
            ],
            options={
                'verbose_name': 'Price',
                'verbose_name_plural': 'Prices',
            },
        ),
        migrations.CreateModel(
            name='Payment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('transaction', models.CharField(max_length=50, verbose_name='Transaction ID')),
                ('provider', models.CharField(max_length=30, verbose_name='Payment Provider')),
                ('amount', models.FloatField(verbose_name='Amount')),
                ('result', models.JSONField(blank=True, default=dict, null=True, verbose_name='Result')),
                ('success', models.BooleanField(default=False, verbose_name='Successful')),
                ('payee_full_name', models.CharField(max_length=50, verbose_name='Name on Card')),
                ('payee_company', models.CharField(blank=True, max_length=50, null=True, verbose_name='Company')),
                ('billing_address', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='vendor.address', verbose_name='payments')),
                ('invoice', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='payments', to='vendor.invoice', verbose_name='Invoice')),
                ('profile', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='payments', to='vendor.customerprofile', verbose_name='Purchase Profile')),
            ],
        ),
        migrations.AddField(
            model_name='address',
            name='profile',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='addresses', to='vendor.customerprofile', verbose_name='Customer Profile'),
        ),
    ]



from django.urls import path

from vendor.views import vendor_admin as admin_views
from vendor.views import report as report_views

app_name = "vendor_admin"

urlpatterns = [
    path('', admin_views.AdminDashboardView.as_view(), name="manager-dashboard"),
    path('products/', admin_views.AdminProductListView.as_view(), name="manager-product-list"),
    path('product/<uuid:uuid>/', admin_views.AdminProductUpdateView.as_view(), name="manager-product-update"),
    path('product/', admin_views.AdminProductCreateView.as_view(), name="manager-product-create"),
    path('offers/', admin_views.AdminOfferListView.as_view(), name="manager-offer-list"),
    path('offer/<uuid:uuid>/', admin_views.AdminOfferUpdateView.as_view(), name="manager-offer-update"),
    path('offer/', admin_views.AdminOfferCreateView.as_view(), name="manager-offer-create"),
    path('orders/', admin_views.AdminInvoiceListView.as_view(), name="manager-order-list"),
    path('order/<uuid:uuid>/', admin_views.AdminInvoiceDetailView.as_view(), name="manager-order-detail"),
    path('subscriptions/', admin_views.AdminSubscriptionListView.as_view(), name="manager-subscriptions"),
    path('subscription/<uuid:uuid>', admin_views.AdminSubscriptionDetailView.as_view(), name="manager-subscription"),
    path('profiles/', admin_views.AdminProfileListView.as_view(), name="manager-profiles"),
    path('profile/<uuid:uuid>', admin_views.AdminProfileDetailView.as_view(), name="manager-profile"),
    path('product/<uuid:uuid>/renew', admin_views.AdminManualSubscriptionRenewal.as_view(), name="manager-product-renew"),
    path('payments/no/receipt/', admin_views.PaymentWithNoReceiptListView.as_view(), name="manager-payment-no-receipt"),
    path('payments/no/orderitems/', admin_views.PaymentWithNoOrderItemsListView.as_view(), name="manager-payment-no-receipt"),
    path("processors/", admin_views.PaymentProcessorSiteConfigsListView.as_view(), name="vendor-processor-lists"),
    path("processor/siteconfig/", admin_views.PaymentProcessorFormView.as_view(), name="vendor-processor"),
    path("processor/siteconfig/<int:pk>/site/", admin_views.PaymentProcessorSiteSelectFormView.as_view(), name="vendor-processor-site"),
    path("authorizenet/integration/", admin_views.AuthorizeNetIntegrationView.as_view(), name="authorizenet-integration"),

    # reports
    path('reports/receipts/download/', report_views.ReceiptListCSV.as_view(), name="manager-receipt-download"),
    path('reports/invoices/download/', report_views.InvoiceListCSV.as_view(), name="manager-invoice-download"),
]



import uuid

from django.db.models.aggregates import Sum
from django.db.models import Count
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.db import models
from django.utils.translation import gettext_lazy as _

from vendor.models.receipt import Receipt
from vendor.models.subscription import Subscription
from vendor.models.base import SoftDeleteModelBase
from vendor.models.choice import PurchaseStatus
from vendor.utils import get_display_decimal



class PaymentReportModelManager(models.Manager):
    def get_total_settled_purchases(self, start_date=None, end_date=None):
        qs = super().get_queryset()

        if not (start_date and end_date):
            return qs.filter(status=PurchaseStatus.SETTLED).aggregate(Sum('amount'))

        if start_date and end_date is None:
            return qs.filter(submitted_date__gte=start_date, status=PurchaseStatus.SETTLED).aggregate(Sum('amount'))

        if start_date is None and end_date:
            return qs.filter(submitted_date__lte=end_date, status=PurchaseStatus.SETTLED).aggregate(Sum('amount'))

        return qs.filter(submitted_date__gte=start_date, submitted_date__lte=end_date, status=PurchaseStatus.SETTLED).aggregate((Sum('amount')))

    def get_total_settled_purchases_by_site(self, site, start_date=None, end_date=None):
        qs = super().get_queryset()

        if not (start_date and end_date):
            amount_qs = qs.filter(profile__site=site, status=PurchaseStatus.SETTLED).aggregate(Sum('amount'))

        elif start_date and end_date is None:
            amount_qs = qs.filter(profile__site=site, submitted_date__gte=start_date, status=PurchaseStatus.SETTLED).aggregate(Sum('amount'))

        elif start_date is None and end_date:
            amount_qs = qs.filter(profile__site=site, submitted_date__lte=end_date, status=PurchaseStatus.SETTLED).aggregate(Sum('amount'))

        else:
            amount_qs = qs.filter(profile__site=site, submitted_date__gte=start_date, submitted_date__lte=end_date, status=PurchaseStatus.SETTLED).aggregate(Sum('amount'))
        
        if not amount_qs['amount__sum']:
            return 0

        return amount_qs['amount__sum']

    
    def get_total_settled_purchases_by_subscription(self, site, start_date=None, end_date=None):
        qs = super().get_queryset()
        organized_data = {}

        if not (start_date and end_date):
            filtered_data = qs.filter(profile__site=site, status=PurchaseStatus.SETTLED).values('subscription').annotate(Sum('amount'))

        elif start_date and end_date is None:
            filtered_data = qs.filter(profile__site=site, submitted_date__gte=start_date, status=PurchaseStatus.SETTLED).values('subscription').annotate(Sum('amount'))

        elif start_date is None and end_date:
            filtered_data = qs.filter(profile__site=site, submitted_date__lte=end_date, status=PurchaseStatus.SETTLED).values('subscription').annotate(Sum('amount'))
        else:
            filtered_data = qs.filter(profile__site=site, submitted_date__gte=start_date, submitted_date__lte=end_date, status=PurchaseStatus.SETTLED).values('subscription').annotate((Sum('amount')))
        
        organized_data = {}
        for data in filtered_data:
            organized_data[Subscription.objects.get(pk=data['subscription']).name] = 0
        
        for data in filtered_data:
            organized_data[Subscription.objects.get(pk=data['subscription']).name] += data['amount__sum']

        return organized_data
    
    def get_total_settled_purchases_by_site_and_subscription(self, start_date=None, end_date=None):
        qs = super().get_queryset()
        organized_data = {}
        
        if not (start_date and end_date):
            filtered_data = qs.filter(status=PurchaseStatus.SETTLED).values('profile__site', 'subscription').annotate(Sum('amount'))

        elif start_date and end_date is None:
            filtered_data = qs.filter(submitted_date__gte=start_date, status=PurchaseStatus.SETTLED).values('profile__site', 'subscription').annotate(Sum('amount'))

        elif start_date is None and end_date:
            filtered_data = qs.filter(submitted_date__lte=end_date, status=PurchaseStatus.SETTLED).values('profile__site', 'subscription').annotate(Sum('amount'))
        
        else:
            filtered_data = qs.filter(submitted_date__gte=start_date, submitted_date__lte=end_date, status=PurchaseStatus.SETTLED).values('profile__site', 'subscription').annotate((Sum('amount')))

        organized_data = {data['profile__site']: {} for data in filtered_data}
        
        for data in filtered_data:
            organized_data[data['profile__site']][Subscription.objects.get(pk=data['subscription']).name] = 0

        for data in filtered_data:
            organized_data[data['profile__site']][Subscription.objects.get(pk=data['subscription']).name] += data['amount__sum']

        return organized_data





##########
# PAYMENT
##########
class Payment(SoftDeleteModelBase):
    '''
    Payments
    - Payments are typically from a Credit Card, PayPal or ACH
    - Multiple Payments can be applied to an invoice
    - Gift cards can be used as payments
    - Discounts are Payment credits
    '''
    uuid = models.UUIDField(_("UUID"), editable=False, unique=True, default=uuid.uuid4, null=False, blank=False)
    invoice = models.ForeignKey("vendor.Invoice", verbose_name=_("Invoice"), on_delete=models.CASCADE, related_name="payments")
    created = models.DateTimeField(_("Date Created"), auto_now_add=True)
    submitted_date = models.DateTimeField(_("Payment Date"), default=None, blank=True, null=True)
    transaction = models.CharField(_("Transaction ID"), max_length=80, blank=True, null=True)
    provider = models.CharField(_("Payment Provider"), max_length=30)
    amount = models.FloatField(_("Amount"))
    profile = models.ForeignKey("vendor.CustomerProfile", verbose_name=_("Purchase Profile"), blank=True, on_delete=models.CASCADE, related_name="payments")
    billing_address = models.ForeignKey("vendor.Address", verbose_name=_("Billing Address"), on_delete=models.CASCADE, blank=True, null=True)
    result = models.JSONField(_("Result"), default=dict, blank=True, null=True)
    success = models.BooleanField(_("Successful"), default=False)
    payee_full_name = models.CharField(_("Name on Card"), max_length=50)
    payee_company = models.CharField(_("Company"), max_length=50, blank=True, null=True)
    status = models.IntegerField(_("Status"), choices=PurchaseStatus.choices, default=0)
    subscription = models.ForeignKey("vendor.Subscription", verbose_name=_("Subscription"), on_delete=models.CASCADE, related_name="payments", blank=True, null=True, default=None)

    reports = PaymentReportModelManager()

    def __str__(self):
        return f"{self.transaction} - {self.profile.user.username}"

    def get_related_receipts(self):
        return Receipt.objects.filter(transaction=self.transaction)

    def get_receipt(self):
        try:
            return Receipt.objects.get(transaction=self.transaction, created__year=self.created.year, created__month=self.created.month, created__day=self.created.day)
        except ObjectDoesNotExist:
            return None
        except MultipleObjectsReturned:
            return Receipt.objects.filter(transaction=self.transaction, created__year=self.created.year, created__month=self.created.month, created__day=self.created.day).first()

    def get_amount_display(self):
        return get_display_decimal(self.amount)
